function [Z, W] = lobatto(i)

%=================================================
% Zeros of the ith-degree Lobatto polynomial
% and corresponding weights for the Lobatto
% integration quadrature
%
% This table contains values for i = 1, 2, ..., 6
% The default value is i=6
%=================================================

%------
% check
%------

if(i>6)

  disp(' -->');
  disp(' lobatto: Chosen number of points');
  disp('   is not available; Will take i=6');
  i=6;

end

%-------
if(i==1)
%-------

Z(1) = 0.0; W(1) = 4.0/3.0;

%-----------
elseif(i==2)
%-----------

Z(1) = -1.0/sqrt(5.0); Z(2) = -Z(1);
W(1) = 5.0/6.0; W(2) = W(1);

%-----------
elseif(i==3)
%-----------

Z(1) = -sqrt(3.0/7.0); Z(2) = 0.0; Z(3) = -Z(1);
W(1) = 49.0/90.0; W(2) = 32.0/45.0; W(3) = W(1);

fc = 2.0/((i+1)*(i+2));
W(1) = fc * 64/ (35*Z(1)^4-30*Z(1)^2+3)^2;
W(2) = fc * 64/ (35*Z(2)^4-30*Z(2)^2+3)^2;
W(3) = W(1);

%-----------
elseif(i==4)
%-----------

Z(1) = -0.76505532392946; Z(2) = -0.28523151648064;
Z(3) = -Z(2); Z(4) = -Z(1);

W(1) = 0.37847495629785; W(2) = 0.55485837703549;
W(3) = W(2); W(4) = W(1);

%fc = 2.0/((i+1)*(i+2));
%W(1) = fc * 64/ ( (63*Z(1)^4-70*Z(1)^2+15)^2 * Z(1)^2);
%W(2) = fc * 64/ ( (63*Z(2)^4-70*Z(2)^2+15)^2 * Z(2)^2);
%W(3) = W(2); W(4) = W(1);
      
%-----------
elseif(i==5)
%-----------

Z(1) = -0.83022389627857; Z(2) = -0.46884879347071;
Z(3) = 0.0; Z(4) = -Z(2); Z(5) = -Z(1);

W(1) = 0.27682604736157; W(2) = 0.43174538120986;
W(3) = 0.48761904761905;
W(4) = W(2); W(5) = W(1);

%fc = 2.0/((i+1)*(i+2));
%W(1) = fc * 256/ ( 231*Z(1)^6-315*Z(1)^4+105*Z(1)^2-5)^2;
%W(2) = fc * 256/ ( 231*Z(2)^6-315*Z(2)^4+105*Z(2)^2-5)^2;
%W(3) = fc * 256/ ( 231*Z(3)^6-315*Z(3)^4+105*Z(3)^2-5)^2;
%W(4) = W(2); W(5) = W(1);

%-----------
elseif(i==6)
%-----------

Z(1) = -0.87174014850961; Z(2) = -0.59170018143314; 
Z(3) = -0.20929921790248;
Z(4) = -Z(3); Z(5) =-Z(2); Z(6) =-Z(1);

W(1) = 0.21070422714350; W(2) = 0.34112269248350;
W(3) = 0.41245879465870;
W(4) = W(3); W(5) = W(2); W(6) = W(1);

%fc = 2.0/((i+1)*(i+2));
%W(1) = fc * 256/ ( ( 429*Z(1)^6-693*Z(1)^4+315*Z(1)^2-35)^2 * Z(1)^2);
%W(2) = fc * 256/ ( ( 429*Z(2)^6-693*Z(2)^4+315*Z(2)^2-35)^2 * Z(2)^2);
%W(3) = fc * 256/ ( ( 429*Z(3)^6-693*Z(3)^4+315*Z(3)^2-35)^2 * Z(3)^2);
%W(4) = W(3); W(5) = W(2); W(6) = W(1);

%--
end
%--

return;
